@@ -22,13 +22,6 @@ impl TreeSitterParser {
             }
         }
 
-        let mut build = cc::Build::new();
-        build.include(&dir).warnings(false); // ignore unused parameter warnings
-        for file in c_files {
-            build.file(dir.join(file));
-        }
-        build.compile(self.name);
-
         if !cpp_files.is_empty() {
             let mut cpp_build = cc::Build::new();
             cpp_build
@@ -58,6 +51,13 @@ impl TreeSitterParser {
             }
             cpp_build.compile(&format!("{}-cpp", self.name));
         }
+
+        let mut build = cc::Build::new();
+        build.include(&dir).warnings(false); // ignore unused parameter warnings
+        for file in c_files {
+            build.file(dir.join(file));
+        }
+        build.compile(self.name);
     }
 }
 
@@ -88,6 +88,11 @@ fn main() {
             src_dir: "vendor/tree-sitter-clojure-src",
             extra_files: vec![],
         },
+        TreeSitterParser {
+            name: "tree-sitter-cmake",
+            src_dir: "vendor/tree-sitter-cmake-src",
+            extra_files: vec!["scanner.cc"],
+        },
         TreeSitterParser {
             name: "tree-sitter-commonlisp",
             src_dir: "vendor/tree-sitter-commonlisp-src",
