@@ -14,7 +14,7 @@ use crate::{
 };
 use Syntax::*;
 
-/// A Debug implementation that does not reucurse into the
+/// A Debug implementation that does not recurse into the
 /// corresponding node mentioned for Unchanged. Otherwise we will
 /// infinitely loop on unchanged nodes, which both point to the other.
 impl<'a> fmt::Debug for ChangeKind<'a> {
@@ -52,6 +52,7 @@ pub struct SyntaxInfo<'a> {
     prev_is_contiguous: Cell<bool>,
     /// The number of nodes that are ancestors of this one.
     num_ancestors: Cell<u32>,
+    pub num_after: Cell<usize>,
     /// A number that uniquely identifies this syntax node.
     unique_id: Cell<SyntaxId>,
     /// A number that uniquely identifies the content of this syntax
@@ -60,6 +61,9 @@ pub struct SyntaxInfo<'a> {
     ///
     /// Values are sequential, not hashes. Collisions never occur.
     content_id: Cell<u32>,
+    /// Is this the only node with this content? Ignores nodes on the
+    /// other side.
+    content_is_unique: Cell<bool>,
 }
 
 impl<'a> SyntaxInfo<'a> {
@@ -71,8 +75,10 @@ impl<'a> SyntaxInfo<'a> {
             parent: Cell::new(None),
             prev_is_contiguous: Cell::new(false),
             num_ancestors: Cell::new(0),
+            num_after: Cell::new(0),
             unique_id: Cell::new(NonZeroU32::new(u32::MAX).unwrap()),
             content_id: Cell::new(0),
+            content_is_unique: Cell::new(false),
         }
     }
 }
@@ -277,6 +283,10 @@ impl<'a> Syntax<'a> {
         self.info().content_id.get()
     }
 
+    pub fn content_is_unique(&self) -> bool {
+        self.info().content_is_unique.get()
+    }
+
     pub fn num_ancestors(&self) -> u32 {
         self.info().num_ancestors.get()
     }
@@ -334,12 +344,15 @@ pub fn init_all_info<'a>(lhs_roots: &[&'a Syntax<'a>], rhs_roots: &[&'a Syntax<'
 
 fn init_info<'a>(lhs_roots: &[&'a Syntax<'a>], rhs_roots: &[&'a Syntax<'a>]) {
     let mut id = NonZeroU32::new(1).unwrap();
-    init_info_single(lhs_roots, &mut id);
-    init_info_single(rhs_roots, &mut id);
+    init_info_on_side(lhs_roots, &mut id);
+    init_info_on_side(rhs_roots, &mut id);
 
     let mut existing = HashMap::new();
     set_content_id(lhs_roots, &mut existing);
     set_content_id(rhs_roots, &mut existing);
+
+    set_content_is_unique(lhs_roots);
+    set_content_is_unique(rhs_roots);
 }
 
 type ContentKey = (Option<String>, Option<String>, Vec<u32>, bool, bool);
@@ -395,6 +408,16 @@ fn set_content_id(nodes: &[&Syntax], existing: &mut HashMap<ContentKey, u32>) {
     }
 }
 
+fn set_num_after(nodes: &[&Syntax], parent_num_after: usize) {
+    for (i, node) in nodes.iter().enumerate() {
+        let num_after = parent_num_after + nodes.len() - 1 - i;
+        node.info().num_after.set(num_after);
+
+        if let List { children, .. } = node {
+            set_num_after(children, num_after);
+        }
+    }
+}
 pub fn init_next_prev<'a>(roots: &[&'a Syntax<'a>]) {
     set_prev_sibling(roots);
     set_next_sibling(roots);
@@ -402,13 +425,16 @@ pub fn init_next_prev<'a>(roots: &[&'a Syntax<'a>]) {
     set_prev_is_contiguous(roots);
 }
 
-fn init_info_single<'a>(roots: &[&'a Syntax<'a>], next_id: &mut SyntaxId) {
+/// Set all the `SyntaxInfo` values for all the `roots` on a single
+/// side (LHS or RHS).
+fn init_info_on_side<'a>(roots: &[&'a Syntax<'a>], next_id: &mut SyntaxId) {
     set_parent(roots, None);
     set_num_ancestors(roots, 0);
+    set_num_after(roots, 0);
     set_unique_id(roots, next_id);
 }
 
-fn set_unique_id<'a>(nodes: &[&'a Syntax<'a>], next_id: &mut SyntaxId) {
+fn set_unique_id(nodes: &[&Syntax], next_id: &mut SyntaxId) {
     for node in nodes {
         node.info().unique_id.set(*next_id);
         *next_id = NonZeroU32::new(u32::from(*next_id) + 1)
@@ -419,6 +445,35 @@ fn set_unique_id<'a>(nodes: &[&'a Syntax<'a>], next_id: &mut SyntaxId) {
     }
 }
 
+/// Assumes that `set_content_id` has already run.
+fn find_nodes_with_unique_content(nodes: &[&Syntax], counts: &mut HashMap<u32, usize>) {
+    for node in nodes {
+        *counts.entry(node.content_id()).or_insert(0) += 1;
+        if let List { children, .. } = node {
+            find_nodes_with_unique_content(children, counts);
+        }
+    }
+}
+
+fn set_content_is_unique_from_counts(nodes: &[&Syntax], counts: &HashMap<u32, usize>) {
+    for node in nodes {
+        let count = counts
+            .get(&node.content_id())
+            .expect("Count should be present");
+        node.info().content_is_unique.set(*count == 1);
+
+        if let List { children, .. } = node {
+            set_content_is_unique_from_counts(children, counts);
+        }
+    }
+}
+
+fn set_content_is_unique(nodes: &[&Syntax]) {
+    let mut counts = HashMap::new();
+    find_nodes_with_unique_content(nodes, &mut counts);
+    set_content_is_unique_from_counts(nodes, &counts);
+}
+
 fn set_prev_sibling<'a>(nodes: &[&'a Syntax<'a>]) {
     for (i, node) in nodes.iter().enumerate() {
         if i == 0 {
@@ -467,7 +522,7 @@ fn set_parent<'a>(nodes: &[&'a Syntax<'a>], parent: Option<&'a Syntax<'a>>) {
     }
 }
 
-fn set_num_ancestors<'a>(nodes: &[&Syntax<'a>], num_ancestors: u32) {
+fn set_num_ancestors(nodes: &[&Syntax], num_ancestors: u32) {
     for node in nodes {
         node.info().num_ancestors.set(num_ancestors);
 
@@ -477,7 +532,7 @@ fn set_num_ancestors<'a>(nodes: &[&Syntax<'a>], num_ancestors: u32) {
     }
 }
 
-fn set_prev_is_contiguous<'a>(roots: &[&Syntax<'a>]) {
+fn set_prev_is_contiguous(roots: &[&Syntax]) {
     for node in roots {
         let is_contiguous = if let Some(prev) = node.info().prev.get() {
             match prev {
